package loveworld.listview3d.util;

/**
 * FlingЧ
 */
public abstract class Dynamics {
	// ===========================================================
	// Constants
	// ===========================================================

	private static final int MAX_TIMESTEP = 50;
	
	// ===========================================================
	// Fields
	// ===========================================================

	/** ǰλ **/
	protected float mPosition;
	/** ǰٶ **/
    protected float mVelocity;
    /** ƶλ **/
    protected float mMaxPosition = Float.MAX_VALUE;
    /** ƶСλ **/
    protected float mMinPosition = -Float.MAX_VALUE;
    /** ¼һθʱʱֵ **/
    protected long mLastTime = 0;

    
	// ===========================================================
	// Constructors
	// ===========================================================

	// ===========================================================
	// Setter
	// ===========================================================
    
    /**
     * óʼ״̬
     * 
     * @param position ǰλ
     * @param velocity ǰٶ
     * @param now      ʱ
     */
    public void setState(final float position, final float velocity, final long now) {
        mVelocity = velocity;
        mPosition = position;
        mLastTime = now;
    }

    /**
     * ƶλ
     * 
     * @param maxPosition
     */
    public void setMaxPosition(final float maxPosition) {
        mMaxPosition = maxPosition;
    }

    
    /**
     * Сƶλ
     * 
     * @param minPosition
     */
    public void setMinPosition(final float minPosition) {
        mMinPosition = minPosition;
    }
    
	// ===========================================================
	// Getter
	// ===========================================================
    
    /**
     * ȡǰλ
     * 
     * @return
     */
    public float getPosition() {
        return mPosition;
    }

    /**
     * ȡǰٶ
     * 
     * @return
     */
    public float getVelocity() {
        return mVelocity;
    }

    
    /**
     * Ƿڻָ״̬
     * 
     * @param velocityTolerance
     * @param positionTolerance
     * @return
     */
    public boolean isAtRest(final float velocityTolerance, final float positionTolerance) {
        final boolean standingStill = Math.abs(mVelocity) < velocityTolerance;
        final boolean withinLimits = mPosition - positionTolerance < mMaxPosition
                && mPosition + positionTolerance > mMinPosition;
        return standingStill && withinLimits;
    }


    /**
     * 
     * 
     * @param now
     */
    public void update(final long now) {
        int dt = (int)(now - mLastTime);
        if (dt > MAX_TIMESTEP) {
            dt = MAX_TIMESTEP;
        }

        onUpdate(dt);

        mLastTime = now;
    }

    
    /**
     * ȡƺĿƶ
     * 
     * @return
     */
    protected float getDistanceToLimit() {
        float distanceToLimit = 0;

        if (mPosition > mMaxPosition) {
            distanceToLimit = mMaxPosition - mPosition;
        } else if (mPosition < mMinPosition) {
            distanceToLimit = mMinPosition - mPosition;
        }

        return distanceToLimit;
    }
    
	// ===========================================================
	// Methods for/from SuperClass/Interfaces
	// ===========================================================

    /**
     * @param dt
     */
    abstract protected void onUpdate(int dt);
    
	// ===========================================================
	// Private Methods
	// ===========================================================

	// ===========================================================
	// Inner and Anonymous Classes
	// ===========================================================

 
    

}
